#!/usr/bin/env python
# encoding: utf-8
"""
@version: Python 3.7
@author: BingZhen Zhou
@contact: 953129171@qq.com
@file name: DG1Dtime.py
@time: 2019/7/31  20:44
"""

"""
DG method solve eq.
$u_t + u_x = 0， x\in[0,1], t>0$
$u(x,0)=\sin(2\pi x)$
Periodic boundary conditions
"""
from scipy.sparse import lil_matrix
import numpy as np
nElements = np.array([10,20,40,80,160,320])
nElement = nElements[5]
h = 1.0/nElement
dt = h*h
T = 3.0

ja = 0.5*h
eledof = 2 # since deg = 1 
A = np.array([[0.5, 0.5], [-0.5, 0.5]])
M = ja*np.array([[2.0,1.0],[1.0, 2.0]])/3.0
C = np.array([[0, -1], [0, 0]])
MI = np.mat(M).I
MIA = MI.dot(-1.0*A)
MIC = MI.dot(-1.0*C)

## Assemble
mathbbL = lil_matrix((nElement*eledof, nElement*eledof))
for i in range(nElement):
    mathbbL[i*eledof:(i+1)*eledof, i*eledof:(i+1)*eledof] = MIA
for i in range(1, nElement):
    mathbbL[i*eledof:(i+1)*eledof, (i-1)*eledof:i*eledof] = MIC
mathbbL[:eledof, -eledof:] = MIC
##----------------------------

## initialize
ux0 = lambda x: np.sin(2*np.pi*np.array(x))
uhn = np.zeros(nElement*eledof)
for ic in range(nElement):
    ux0ele = ux0([ic*h, (ic+1)*h])
    uhn[ic*eledof:(ic+1)*eledof] = ux0ele
##----------------------------


## iteration
step = 0
maxstep = int(T/dt)
while(step < maxstep):
    uhn = uhn + dt*mathbbL.dot(uhn)
    step += 1
##----------------------------

## error
u = lambda x: np.sin(2*np.pi*(np.array(x) - T))
pts, wts = np.polynomial.legendre.leggauss(8)
def Looerror(uh, u):
    max = 0
    for ic in range(nElement):
        L, R = ic*h, (ic+1)*h
        xele = 0.5*(L+R) + 0.5*(R-L)*pts
        ux = u(xele)
        uhele = uh[ic*eledof: (ic+1)*eledof]
        tmp = ux - np.interp(pts, [-1, 1], uhele)
        tmp = abs(tmp)
        if(max <= tmp.max()):
            max = tmp.max()
    return max
    
def L2error(uh, u):
    sum = 0.0
    for ic in range(nElement):
        L, R = ic*h, (ic+1)*h
        xele = 0.5*(L+R) + 0.5*(R-L)*pts
        ux = u(xele)
        uhele = uh[ic*eledof: (ic+1)*eledof]
        tmp = ux - np.interp(pts, [-1, 1], uhele)
        sum += (tmp*tmp*wts).sum()*h*0.5
    return np.sqrt(sum)
    
print(Looerror(uhn, u))
print(L2error(uhn, u))

"""
	loo		l2	
0.1	    7.3889E-01	        	4.5861E-01	
0.05	1.5734E-01	2.23151 	1.0041E-01	2.19138 
0.025	3.8635E-02	2.02589 	2.4600E-02	2.02918 
0.0125	9.6536E-03	2.00075 	6.1613E-03	1.99735 
0.06125	2.4213E-03	1.99530 	1.5465E-03	1.99424 
0.030625 6.0675E-04	1.99658 	3.8770E-04	1.99599 
"""

"""
import matplotlib.pyplot as plt
fig = plt.figure(figsize=(8,8), dpi=72,facecolor="white")
axes = plt.subplot(111)
plt.axis([0, 1, -1.1, 1.1])
for ic in range(nElement):
    xele = [ic*h, (ic+1)*h]
    uhele = uhn[ic*eledof: (ic+1)*eledof]
    plt.plot(xele, uhele)
    plt.plot(xele, u(xele), color='red')
plt.show()
"""